@@ -41,24 +41,33 @@
   } else if (inherits(model, "geeglm")) {
     res_ <- stats::residuals(model, type = "pearson")
   } else if (inherits(model, "glm")) {
-    res_ <- stats::rstandard(model, type = "pearson")
+    res_ <- .safe(abs(stats::rstandard(model, type = "deviance")))
   } else {
-    res_ <- tryCatch(stats::rstudent(model), error = function(e) NULL)
+    res_ <- .safe(stats::rstudent(model))
     if (is.null(res_)) {
-      res_ <- tryCatch(stats::residuals(model), error = function(e) NULL)
+      res_ <- .safe(stats::residuals(model))
     }
   }
 
   if (is.null(res_)) {
     if (verbose) {
-      insight::format_alert(sprintf("QQ plot could not be created. Cannot extract residuals from objects of class `%s`.", class(model)[1]))
+      insight::format_alert(
+        sprintf(
+          "QQ plot could not be created. Cannot extract residuals from objects of class `%s`.",
+          class(model)[1]
+        )
+      )
     }
     return(NULL)
   }
 
-  fitted_ <- stats::fitted(model)
+  if (inherits(model, "glm")) {
+    fitted_ <- stats::qnorm((stats::ppoints(length(res_)) + 1) / 2)
+  } else {
+    fitted_ <- stats::fitted(model)
+  }
 
-  # sanity check, sometime either residuals or fitted can contain NA, see #488
+  # validation check, sometimes either residuals or fitted can contain NA, see #488
   if (anyNA(res_) || anyNA(fitted_)) {
     # drop NA and make sure both fitted and residuals match
     non_na <- !is.na(fitted_) & !is.na(res_)
@@ -76,9 +85,9 @@
 
 # prepare data for random effects QQ plot ----------------------------------
 
-.diag_reqq <- function(model, level = 0.95, model_info, verbose = TRUE) {
+.diag_reqq <- function(model, level = 0.95, model_info = NULL, verbose = TRUE) {
   # check if we have mixed model
-  if (!model_info$is_mixed) {
+  if (is.null(model_info) || !model_info$is_mixed) {
     return(NULL)
   }
 
@@ -121,7 +130,7 @@
   }
 
 
-  mapply(function(.re, .se) {
+  Map(function(.re, .se) {
     ord <- unlist(lapply(.re, order)) + rep((0:(ncol(.re) - 1)) * nrow(.re), each = nrow(.re))
 
     df.y <- unlist(.re)[ord]
@@ -136,7 +145,7 @@
       stringsAsFactors = FALSE,
       row.names = NULL
     )
-  }, re, se, SIMPLIFY = FALSE)
+  }, re, se)
 }
 
 
@@ -144,10 +153,13 @@
 # prepare data for normality of residuals plot ----------------------------------
 
 .diag_norm <- function(model, verbose = TRUE) {
-  r <- try(stats::residuals(model), silent = TRUE)
+  r <- try(as.numeric(stats::residuals(model)), silent = TRUE)
 
   if (inherits(r, "try-error")) {
-    insight::format_alert(sprintf("Non-normality of residuals could not be computed. Cannot extract residuals from objects of class '%s'.", class(model)[1]))
+    insight::format_alert(sprintf(
+      "Non-normality of residuals could not be computed. Cannot extract residuals from objects of class '%s'.",
+      class(model)[1]
+    ))
     return(NULL)
   }
 
@@ -165,16 +177,16 @@
 
   if (inherits(model, "lm", which = TRUE) == 1) {
     cook_levels <- round(stats::qf(0.5, s$fstatistic[2], s$fstatistic[3]), 2)
-  } else if (!is.null(threshold)) {
-    cook_levels <- threshold
-  } else {
+  } else if (is.null(threshold)) {
     cook_levels <- c(0.5, 1)
+  } else {
+    cook_levels <- threshold
   }
 
   n_params <- tryCatch(model$rank, error = function(e) insight::n_parameters(model))
 
   infl <- stats::influence(model, do.coef = FALSE)
-  resid <- insight::get_residuals(model)
+  resid <- as.numeric(insight::get_residuals(model))
 
   std_resid <- tryCatch(stats::rstandard(model, infl), error = function(e) resid)
 
@@ -203,8 +215,8 @@
   ncv <- tryCatch(
     {
       data.frame(
-        x = stats::fitted(model),
-        y = stats::residuals(model)
+        x = as.numeric(stats::fitted(model)),
+        y = as.numeric(stats::residuals(model))
       )
     },
     error = function(e) {
@@ -214,7 +226,10 @@
 
   if (is.null(ncv)) {
     if (verbose) {
-      message(insight::format_message(sprintf("Non-constant error variance could not be computed. Cannot extract residuals from objects of class '%s'.", class(model)[1])))
+      insight::format_alert(sprintf(
+        "Non-constant error variance could not be computed. Cannot extract residuals from objects of class '%s'.",
+        class(model)[1]
+      ))
     }
     return(NULL)
   }
@@ -242,6 +257,7 @@
         }
         stats::residuals(model) / sigma
       } else if (inherits(model, "glm")) {
+        ## TODO: check if we can / should use deviance residuals (as for QQ plots) here as well?
         stats::rstandard(model, type = "pearson")
       } else {
         stats::rstandard(model)
@@ -254,7 +270,10 @@
 
   if (is.null(r)) {
     if (verbose) {
-      message(insight::format_message(sprintf("Homogeneity of variance could not be computed. Cannot extract residual variance from objects of class '%s'.", class(model)[1])))
+      insight::format_alert(sprintf(
+        "Homogeneity of variance could not be computed. Cannot extract residual variance from objects of class '%s'.",
+        class(model)[1]
+      ))
     }
     return(NULL)
   }
@@ -317,7 +336,7 @@
     }
     d$Prob <- stats::predict(model, type = ptype)
     d$Disp <- insight::get_sigma(model)
-    d$V <- d$Predicted * (1 + d$Predicted / d$Disp) * (1 - d$Prob) * (1 + d$Predicted * (1 + d$Predicted / d$Disp) * d$Prob)
+    d$V <- d$Predicted * (1 + d$Predicted / d$Disp) * (1 - d$Prob) * (1 + d$Predicted * (1 + d$Predicted / d$Disp) * d$Prob) # nolint
     d$StdRes <- insight::get_residuals(model, type = "pearson")
   }
 
@@ -333,7 +352,7 @@
     }
     d$Prob <- stats::predict(model, type = ptype)
     d$Disp <- stats::predict(model, type = "disp")
-    d$V <- d$Predicted * (1 + d$Predicted / d$Disp) * (1 - d$Prob) * (1 + d$Predicted * (1 + d$Predicted / d$Disp) * d$Prob)
+    d$V <- d$Predicted * (1 + d$Predicted / d$Disp) * (1 - d$Prob) * (1 + d$Predicted * (1 + d$Predicted / d$Disp) * d$Prob) # nolint
     d$StdRes <- insight::get_residuals(model, type = "pearson")
   }
 
