@@ -4,12 +4,14 @@
 #' @description Prints tables (i.e. data frame) in different output formats.
 #'   `print_md()` is a alias for `display(format = "markdown")`.
 #'
-#' @param object,x An object returned by [`model_performance()`][model_performance]
-#'   or [`compare_performance()`][compare_performance].
+#' @param object,x An object returned by [`model_performance()`] or
+#'   or [`compare_performance()`].
 #'   or its summary.
 #' @param format String, indicating the output format. Currently, only
 #'   `"markdown"` is supported.
+#' @param layout Table layout (can be either `"horizontal"` or `"vertical"`).
 #' @param digits Number of decimal places.
+#' @param caption Table caption as string. If `NULL`, no table caption is printed.
 #' @param ... Currently not used.
 #'
 #' @return A character vector. If `format = "markdown"`, the return value
@@ -27,8 +29,12 @@
 #' mp <- model_performance(model)
 #' display(mp)
 #' @export
-display.performance_model <- function(object, format = "markdown", digits = 2, ...) {
-  print_md(x = object, digits = digits, ...)
+display.performance_model <- function(object, format = "markdown", digits = 2, caption = NULL, ...) {
+  if (identical(format, "html")) {
+    print_html(x = object, digits = digits, caption = caption, ...)
+  } else {
+    print_md(x = object, digits = digits, caption = caption, ...)
+  }
 }
 
 
