@@ -29,7 +29,7 @@ performance_hosmer <- function(model, n_bins = 10) {
 
   # check for valid object class
   if (!inherits(model, c("glmerMod", "glm"))) {
-    stop("`model` must be an object of class 'glm' or 'glmerMod'.", call. = FALSE)
+    insight::format_error("`model` must be an object of class `glm` or `glmerMod`.")
   }
 
   # mixed models (lme4)
