@@ -6,11 +6,12 @@
 #' @inheritParams performance_rmse
 #' @inheritParams model_performance.lm
 #'
-#' @details The mean square error is the mean of the sum of squared residuals,
-#'   i.e. it measures the average of the squares of the errors. Less technically
-#'   speaking, the mean square error can be considered as the variance of the
-#'   residuals, i.e. the variation in the outcome the model doesn't explain.
-#'   Lower values (closer to zero) indicate better fit.
+#' @details
+#' The mean square error is the mean of the sum of squared residuals, i.e. it
+#' measures the average of the squares of the errors. Less technically speaking,
+#' the mean square error can be considered as the variance of the residuals,
+#' i.e. the variation in the outcome the model doesn't explain. Lower values
+#' (closer to zero) indicate better fit.
 #'
 #' @return Numeric, the mean square error of `model`.
 #'
@@ -31,28 +32,18 @@ mse <- performance_mse
 
 #' @export
 performance_mse.default <- function(model, verbose = TRUE, ...) {
-  res <- tryCatch(
-    {
-      insight::get_residuals(model, verbose = verbose, type = "response", ...)
-    },
-    error = function(e) {
-      NULL
-    }
-  )
+  res <- .safe(insight::get_residuals(model, verbose = verbose, type = "response", ...))
 
   if (is.null(res)) {
-    res <- tryCatch(
-      {
-        def_res <- insight::get_residuals(model, verbose = FALSE, ...)
-        if (verbose) {
-          warning(insight::format_message("Response residuals not available to calculate mean square error. (R)MSE is probably not reliable."), call. = FALSE)
-        }
-        def_res
-      },
-      error = function(e) {
-        NULL
+    res <- .safe({
+      def_res <- insight::get_residuals(model, verbose = FALSE, ...)
+      if (verbose) {
+        insight::format_alert(
+          "Response residuals not available to calculate mean square error. (R)MSE is probably not reliable."
+        )
       }
-    )
+      def_res
+    })
   }
 
   if (is.null(res) || all(is.na(res))) {
@@ -62,7 +53,7 @@ performance_mse.default <- function(model, verbose = TRUE, ...) {
   # for multivariate response models...
   if (is.data.frame(res)) {
     if (verbose) {
-      warning(insight::format_message("Multiple response variables detected. Cannot reliably compute (R)MSE."), call. = FALSE)
+      insight::format_warning("Multiple response variables detected. Cannot reliably compute (R)MSE.")
     }
     return(NA)
   }
