@@ -5,7 +5,7 @@
 #'   including Bayesian regression models.
 #'
 #' @param model A model.
-#' @param normalized Logical, use \code{TRUE} if normalized rmse should be returned.
+#' @param normalized Logical, use `TRUE` if normalized rmse should be returned.
 #' @inheritParams model_performance.lm
 #'
 #' @details The RMSE is the square root of the variance of the residuals and indicates
@@ -13,37 +13,35 @@
 #'   to model's predicted values). It can be interpreted as the standard
 #'   deviation of the unexplained variance, and is in the same units as the
 #'   response variable. Lower values indicate better model fit.
-#'   \cr \cr
+#'
 #'   The normalized RMSE is the proportion of the RMSE related to the
 #'   range of the response variable. Hence, lower values indicate
 #'   less residual variance.
 #'
 #' @return Numeric, the root mean squared error.
 #'
-#' @examples
-#' if (require("nlme")) {
-#'   m <- lme(distance ~ age, data = Orthodont)
+#' @examplesIf require("nlme")
+#' data(Orthodont, package = "nlme")
+#' m <- nlme::lme(distance ~ age, data = Orthodont)
 #'
-#'   # RMSE
-#'   performance_rmse(m, normalized = FALSE)
+#' # RMSE
+#' performance_rmse(m, normalized = FALSE)
 #'
-#'   # normalized RMSE
-#'   performance_rmse(m, normalized = TRUE)
-#' }
-#' @importFrom insight get_response
+#' # normalized RMSE
+#' performance_rmse(m, normalized = TRUE)
 #' @export
 performance_rmse <- function(model, normalized = FALSE, verbose = TRUE) {
   tryCatch(
     {
       # compute rmse
-      rmse_val <- sqrt(performance_mse(model))
+      rmse_val <- sqrt(performance_mse(model, verbose = verbose))
 
       # if normalized, divide by range of response
       if (normalized) {
         # get response
-        resp <- .factor_to_numeric(insight::get_response(model))
-        # cpmpute rmse, normalized
-        rmse_val <- rmse_val / (max(resp, na.rm = T) - min(resp, na.rm = T))
+        resp <- datawizard::to_numeric(insight::get_response(model, verbose = FALSE), dummy_factors = FALSE, preserve_levels = TRUE)
+        # compute rmse, normalized
+        rmse_val <- rmse_val / (max(resp, na.rm = TRUE) - min(resp, na.rm = TRUE))
       }
 
       rmse_val
