@@ -32,7 +32,7 @@
 #'   [`r2_nakagawa()`], [`r2_tjur()`], [`r2_xu()`] and
 #'   [`r2_zeroinflated()`].
 #'
-#' @examples
+#' @examplesIf require("lme4")
 #' # Pseudo r-quared for GLM
 #' model <- glm(vs ~ wt + mpg, data = mtcars, family = "binomial")
 #' r2(model)
@@ -41,21 +41,16 @@
 #' model <- lm(mpg ~ wt + hp, data = mtcars)
 #' r2(model, ci = 0.95)
 #'
-#' if (require("lme4")) {
-#'   model <- lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
-#'   r2(model)
-#' }
+#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
+#' r2(model)
 #' @export
 r2 <- function(model, ...) {
   UseMethod("r2")
 }
 
 
-
-
 # Default models -----------------------------------------------
 
-
 #' @rdname r2
 #' @export
 r2.default <- function(model, ci = NULL, verbose = TRUE, ...) {
@@ -64,7 +59,8 @@ r2.default <- function(model, ci = NULL, verbose = TRUE, ...) {
     return(.r2_ci(model, ci = ci, verbose = verbose, ...))
   }
 
-  if (is.null(minfo <- list(...)$model_info)) {
+  minfo <- list(...)$model_info
+  if (is.null(minfo)) {
     minfo <- suppressWarnings(insight::model_info(model, verbose = FALSE))
   }
 
@@ -113,6 +109,10 @@ r2.lm <- function(model, ci = NULL, ...) {
   .r2_lm(summary(model))
 }
 
+#' @export
+r2.phylolm <- r2.lm
+
+# helper -------------
 
 .r2_lm <- function(model_summary, ci = NULL) {
   out <- list(
@@ -139,7 +139,6 @@ r2.lm <- function(model, ci = NULL, ...) {
 }
 
 
-
 #' @export
 r2.summary.lm <- function(model, ci = NULL, ...) {
   if (!is.null(ci) && !is.na(ci)) {
@@ -149,7 +148,6 @@ r2.summary.lm <- function(model, ci = NULL, ...) {
 }
 
 
-
 #' @export
 r2.systemfit <- function(model, ...) {
   out <- lapply(summary(model)$eq, function(model_summary) {
@@ -172,11 +170,11 @@ r2.systemfit <- function(model, ...) {
 #' @export
 r2.lm_robust <- function(model, ...) {
   out <- list(
-    "R2" = tryCatch(
+    R2 = tryCatch(
       model[["r.squared"]],
       error = function(e) NULL
     ),
-    "R2_adjusted" = tryCatch(
+    R2_adjusted = tryCatch(
       model[["adj.r.squared"]],
       error = function(e) NULL
     )
@@ -197,8 +195,6 @@ r2.ols <- function(model, ...) {
   structure(class = "r2_generic", out)
 }
 
-
-
 #' @export
 r2.lrm <- r2.ols
 
@@ -206,7 +202,6 @@ r2.lrm <- r2.ols
 r2.cph <- r2.ols
 
 
-
 #' @export
 r2.mhurdle <- function(model, ...) {
   resp <- insight::get_response(model, verbose = FALSE)
@@ -229,7 +224,6 @@ r2.mhurdle <- function(model, ...) {
 }
 
 
-
 #' @export
 r2.aov <- function(model, ci = NULL, ...) {
   if (!is.null(ci) && !is.na(ci)) {
@@ -251,7 +245,6 @@ r2.aov <- function(model, ci = NULL, ...) {
 }
 
 
-
 #' @export
 r2.mlm <- function(model, ...) {
   model_summary <- summary(model)
@@ -275,21 +268,21 @@ r2.mlm <- function(model, ...) {
 }
 
 
-
 #' @export
 r2.glm <- function(model, ci = NULL, verbose = TRUE, ...) {
   if (!is.null(ci) && !is.na(ci)) {
     return(.r2_ci(model, ci = ci, verbose = verbose, ...))
   }
 
-  if (is.null(info <- list(...)$model_info)) {
+  info <- list(...)$model_info
+  if (is.null(info)) {
     info <- suppressWarnings(insight::model_info(model, verbose = FALSE))
   }
 
   if (info$family %in% c("gaussian", "inverse.gaussian")) {
     out <- r2.default(model, ...)
   } else if (info$is_logit && info$is_bernoulli) {
-    out <- list("R2_Tjur" = r2_tjur(model, ...))
+    out <- list(R2_Tjur = r2_tjur(model, model_info = info, ...))
     attr(out, "model_type") <- "Logistic"
     names(out$R2_Tjur) <- "Tjur's R2"
     class(out) <- c("r2_pseudo", class(out))
@@ -299,7 +292,7 @@ r2.glm <- function(model, ci = NULL, verbose = TRUE, ...) {
     }
     out <- NULL
   } else {
-    out <- list("R2_Nagelkerke" = r2_nagelkerke(model, ...))
+    out <- list(R2_Nagelkerke = r2_nagelkerke(model, ...))
     names(out$R2_Nagelkerke) <- "Nagelkerke's R2"
     attr(out, "model_type") <- "Generalized Linear"
     class(out) <- c("r2_pseudo", class(out))
@@ -311,6 +304,14 @@ r2.glm <- function(model, ci = NULL, verbose = TRUE, ...) {
 r2.glmx <- r2.glm
 
 
+#' @export
+r2.nestedLogit <- function(model, ci = NULL, verbose = TRUE, ...) {
+  out <- list(R2_Tjur = r2_tjur(model, ...))
+  attr(out, "model_type") <- "Logistic"
+  class(out) <- c("r2_pseudo", class(out))
+  out
+}
+
 
 
 # mfx models ---------------------
@@ -356,7 +357,7 @@ r2.model_fit <- r2.logitmfx
 
 #' @export
 r2.BBreg <- function(model, ...) {
-  out <- list("R2_CoxSnell" = r2_coxsnell(model))
+  out <- list(R2_CoxSnell = r2_coxsnell(model))
   names(out$R2_CoxSnell) <- "Cox & Snell's R2"
   class(out) <- c("r2_pseudo", class(out))
   out
@@ -376,7 +377,7 @@ r2.bayesx <- r2.BBreg
 
 #' @export
 r2.censReg <- function(model, ...) {
-  out <- list("R2_Nagelkerke" = r2_nagelkerke(model))
+  out <- list(R2_Nagelkerke = r2_nagelkerke(model))
   names(out$R2_Nagelkerke) <- "Nagelkerke's R2"
   class(out) <- c("r2_pseudo", class(out))
   out
@@ -394,9 +395,6 @@ r2.clm2 <- r2.censReg
 #' @export
 r2.coxph <- r2.censReg
 
-#' @export
-r2.mclogit <- r2.censReg
-
 #' @export
 r2.polr <- r2.censReg
 
@@ -415,7 +413,15 @@ r2.brmultinom <- r2.censReg
 #' @export
 r2.bife <- r2.censReg
 
+#' @export
+r2.mclogit <- function(model, ...) {
+  r2_nagelkerke(model)
+}
 
+#' @export
+r2.mblogit <- function(model, ...) {
+  r2_nagelkerke(model)
+}
 
 
 
@@ -500,8 +506,8 @@ r2.wbm <- function(model, tolerance = 1e-5, ...) {
     names(r2_marginal) <- "Marginal R2"
 
     out <- list(
-      "R2_conditional" = r2_conditional,
-      "R2_marginal" = r2_marginal
+      R2_conditional = r2_conditional,
+      R2_marginal = r2_marginal
     )
 
     attr(out, "model_type") <- "Fixed Effects"
@@ -524,8 +530,8 @@ r2.sem <- function(model, ...) {
   structure(
     class = "r2_nakagawa",
     list(
-      "R2_conditional" = r2_conditional,
-      "R2_marginal" = r2_marginal
+      R2_conditional = r2_conditional,
+      R2_marginal = r2_marginal
     )
   )
 }
@@ -554,14 +560,12 @@ r2.BFBayesFactor <- r2.brmsfit
 #' @export
 r2.gam <- function(model, ...) {
   # gamlss inherits from gam, and summary.gamlss prints results automatically
-  printout <- utils::capture.output(s <- summary(model))
+  printout <- utils::capture.output(s <- summary(model)) # nolint
 
-  if (!is.null(s$r.sq)) {
-    list(
-      R2 = c(`Adjusted R2` = s$r.sq)
-    )
-  } else {
+  if (is.null(s$r.sq)) {
     NextMethod()
+  } else {
+    list(R2 = c(`Adjusted R2` = s$r.sq))
   }
 }
 
@@ -596,7 +600,7 @@ r2.rma <- function(model, ...) {
 #' @export
 r2.feis <- function(model, ...) {
   out <- list(
-    R2 = c(`R2` = model$r2),
+    R2 = c(R2 = model$r2),
     R2_adjusted = c(`adjusted R2` = model$adj.r2)
   )
 
@@ -636,13 +640,18 @@ r2.fixest <- function(model, ...) {
   structure(class = "r2_generic", out)
 }
 
+#' @export
+r2.fixest_multi <- function(model, ...) {
+  lapply(model, r2.fixest)
+}
+
 
 
 #' @export
 r2.felm <- function(model, ...) {
   model_summary <- summary(model)
   out <- list(
-    R2 = c(`R2` = model_summary$r2),
+    R2 = c(R2 = model_summary$r2),
     R2_adjusted = c(`adjusted R2` = model_summary$r2adj)
   )
 
@@ -656,7 +665,7 @@ r2.felm <- function(model, ...) {
 #' @export
 r2.iv_robust <- function(model, ...) {
   out <- list(
-    R2 = c(`R2` = model$r.squared),
+    R2 = c(R2 = model$r.squared),
     R2_adjusted = c(`adjusted R2` = model$adj.r.squared)
   )
 
@@ -670,7 +679,7 @@ r2.iv_robust <- function(model, ...) {
 r2.ivreg <- function(model, ...) {
   model_summary <- summary(model)
   out <- list(
-    R2 = c(`R2` = model_summary$r.squared),
+    R2 = c(R2 = model_summary$r.squared),
     R2_adjusted = c(`adjusted R2` = model_summary$adj.r.squared)
   )
 
@@ -682,7 +691,7 @@ r2.ivreg <- function(model, ...) {
 
 #' @export
 r2.bigglm <- function(model, ...) {
-  out <- list("R2_CoxSnell" = summary(model)$rsq)
+  out <- list(R2_CoxSnell = summary(model)$rsq)
   names(out$R2_CoxSnell) <- "Cox & Snell's R2"
   class(out) <- c("r2_pseudo", class(out))
   out
@@ -716,7 +725,7 @@ r2.biglm <- function(model, ...) {
 r2.lmrob <- function(model, ...) {
   model_summary <- summary(model)
   out <- list(
-    R2 = c(`R2` = model_summary$r.squared),
+    R2 = c(R2 = model_summary$r.squared),
     R2_adjusted = c(`adjusted R2` = model_summary$adj.r.squared)
   )
 
@@ -735,18 +744,12 @@ r2.mmclogit <- function(model, ...) {
 }
 
 
-#' @export
-r2.mclogit <- function(model, ...) {
-  list(R2 = NA)
-}
-
-
 #' @export
 r2.Arima <- function(model, ...) {
-  if (!requireNamespace("forecast", quietly = TRUE)) {
-    list(R2 = NA)
-  } else {
+  if (requireNamespace("forecast", quietly = TRUE)) {
     list(R2 = stats::cor(stats::fitted(model), insight::get_data(model, verbose = FALSE))^2)
+  } else {
+    list(R2 = NA)
   }
 }
 
@@ -756,8 +759,8 @@ r2.Arima <- function(model, ...) {
 r2.plm <- function(model, ...) {
   model_summary <- summary(model)
   out <- list(
-    "R2" = c(`R2` = model_summary$r.squared[1]),
-    "R2_adjusted" = c(`adjusted R2` = model_summary$r.squared[2])
+    R2 = c(R2 = model_summary$r.squared[1]),
+    R2_adjusted = c(`adjusted R2` = model_summary$r.squared[2])
   )
 
   attr(out, "model_type") <- "Panel Data"
@@ -773,8 +776,8 @@ r2.selection <- function(model, ...) {
     return(NULL)
   }
   out <- list(
-    "R2" = c(`R2` = model_summary$rSquared$R2),
-    "R2_adjusted" = c(`adjusted R2` = model_summary$rSquared$R2adj)
+    R2 = c(R2 = model_summary$rSquared$R2),
+    R2_adjusted = c(`adjusted R2` = model_summary$rSquared$R2adj)
   )
 
   attr(out, "model_type") <- "Tobit 2"
@@ -789,7 +792,7 @@ r2.svyglm <- function(model, ...) {
   rsq.adjust <- 1 - ((1 - rsq) * (model$df.null / model$df.residual))
 
   out <- list(
-    R2 = c(`R2` = rsq),
+    R2 = c(R2 = rsq),
     R2_adjusted = c(`adjusted R2` = rsq.adjust)
   )
 
@@ -801,7 +804,7 @@ r2.svyglm <- function(model, ...) {
 
 #' @export
 r2.vglm <- function(model, ...) {
-  out <- list("R2_McKelvey" = r2_mckelvey(model))
+  out <- list(R2_McKelvey = r2_mckelvey(model))
   names(out$R2_McKelvey) <- "McKelvey's R2"
   class(out) <- c("r2_pseudo", class(out))
   out
@@ -814,7 +817,7 @@ r2.vgam <- r2.vglm
 
 #' @export
 r2.DirichletRegModel <- function(model, ...) {
-  out <- list("R2_Nagelkerke" = r2_nagelkerke(model))
+  out <- list(R2_Nagelkerke = r2_nagelkerke(model))
   names(out$R2_Nagelkerke) <- "Nagelkerke's R2"
   class(out) <- c("r2_pseudo", class(out))
   out
@@ -827,15 +830,13 @@ r2.DirichletRegModel <- function(model, ...) {
 # helper -------------------
 
 .check_r2_ci_args <- function(ci = NULL, ci_method = "bootstrap", valid_ci_method = NULL, verbose = TRUE) {
-  if (!is.null(ci) && !is.na(ci)) {
-    if (!is.null(valid_ci_method) && !ci_method %in% valid_ci_method) {
-      if (verbose) {
-        insight::format_warning(
-          paste0("Method `", ci_method, "` to compute confidence intervals for R2 not supported.")
-        )
-      }
-      return(NULL)
+  if (!is.null(ci) && !is.na(ci) && !is.null(valid_ci_method) && !ci_method %in% valid_ci_method) {
+    if (verbose) {
+      insight::format_warning(
+        paste0("Method `", ci_method, "` to compute confidence intervals for R2 not supported.")
+      )
     }
+    return(NULL)
   }
   ci
 }
