@@ -22,63 +22,78 @@
 #' @details `r2_bayes()` returns an "unadjusted" R2 value. See
 #'   [r2_loo()] to calculate a LOO-adjusted R2, which comes
 #'   conceptually closer to an adjusted R2 measure.
-#'   \cr \cr
+#'
 #'   For mixed models, the conditional and marginal R2 are returned. The marginal
 #'   R2 considers only the variance of the fixed effects, while the conditional
 #'   R2 takes both the fixed and random effects into account.
-#'   \cr \cr
+#'
 #'   `r2_posterior()` is the actual workhorse for `r2_bayes()` and
 #'   returns a posterior sample of Bayesian R2 values.
 #'
-#' @examples
+#' @examplesIf require("rstanarm") && require("rstantools") && require("BayesFactor") && require("brms")
 #' library(performance)
-#' if (require("rstanarm") && require("rstantools")) {
-#'   model <- stan_glm(mpg ~ wt + cyl, data = mtcars, chains = 1, iter = 500, refresh = 0)
-#'   r2_bayes(model)
+#' \donttest{
+#' model <- suppressWarnings(rstanarm::stan_glm(
+#'   mpg ~ wt + cyl,
+#'   data = mtcars,
+#'   chains = 1,
+#'   iter = 500,
+#'   refresh = 0,
+#'   show_messages = FALSE
+#' ))
+#' r2_bayes(model)
 #'
-#'   model <- stan_lmer(
-#'     Petal.Length ~ Petal.Width + (1 | Species),
-#'     data = iris,
-#'     chains = 1,
-#'     iter = 500,
-#'     refresh = 0
-#'   )
-#'   r2_bayes(model)
-#' }
-#' \dontrun{
-#' if (require("BayesFactor")) {
-#'   data(mtcars)
-#'
-#'   BFM <- generalTestBF(mpg ~ qsec + gear, data = mtcars, progress = FALSE)
-#'   FM <- lm(mpg ~ qsec + gear, data = mtcars)
-#'
-#'   r2_bayes(FM)
-#'   r2_bayes(BFM[3])
-#'   r2_bayes(BFM, average = TRUE) # across all models
-#'
-#'
-#'   # with random effects:
-#'   mtcars$gear <- factor(mtcars$gear)
-#'   model <- lmBF(
-#'     mpg ~ hp + cyl + gear + gear:wt,
-#'     mtcars,
-#'     progress = FALSE,
-#'     whichRandom = c("gear", "gear:wt")
-#'   )
-#'   r2_bayes(model)
+#' model <- suppressWarnings(rstanarm::stan_lmer(
+#'   Petal.Length ~ Petal.Width + (1 | Species),
+#'   data = iris,
+#'   chains = 1,
+#'   iter = 500,
+#'   refresh = 0
+#' ))
+#' r2_bayes(model)
 #' }
 #'
-#' if (require("brms")) {
-#'   model <- brms::brm(mpg ~ wt + cyl, data = mtcars)
-#'   r2_bayes(model)
+#' BFM <- BayesFactor::generalTestBF(mpg ~ qsec + gear, data = mtcars, progress = FALSE)
+#' FM <- BayesFactor::lmBF(mpg ~ qsec + gear, data = mtcars)
 #'
-#'   model <- brms::brm(Petal.Length ~ Petal.Width + (1 | Species), data = iris)
-#'   r2_bayes(model)
+#' r2_bayes(FM)
+#' r2_bayes(BFM[3])
+#' r2_bayes(BFM, average = TRUE) # across all models
+#'
+#' # with random effects:
+#' mtcars$gear <- factor(mtcars$gear)
+#' model <- BayesFactor::lmBF(
+#'   mpg ~ hp + cyl + gear + gear:wt,
+#'   mtcars,
+#'   progress = FALSE,
+#'   whichRandom = c("gear", "gear:wt")
+#' )
+#'
+#' r2_bayes(model)
+#'
+#' \donttest{
+#' model <- suppressWarnings(brms::brm(
+#'   mpg ~ wt + cyl,
+#'   data = mtcars,
+#'   silent = 2,
+#'   refresh = 0
+#' ))
+#' r2_bayes(model)
+#'
+#' model <- suppressWarnings(brms::brm(
+#'   Petal.Length ~ Petal.Width + (1 | Species),
+#'   data = iris,
+#'   silent = 2,
+#'   refresh = 0
+#' ))
+#' r2_bayes(model)
 #' }
-#' }
-#' @references Gelman, A., Goodrich, B., Gabry, J., & Vehtari, A. (2018). R-squared for Bayesian regression models. The American Statistician, 1–6. \doi{10.1080/00031305.2018.1549100}
+#' @references
+#' Gelman, A., Goodrich, B., Gabry, J., and Vehtari, A. (2018).
+#' R-squared for Bayesian regression models. The American Statistician, 1–6.
+#' \doi{10.1080/00031305.2018.1549100}
 #' @export
-r2_bayes <- function(model, robust = TRUE, ci = .95, verbose = TRUE, ...) {
+r2_bayes <- function(model, robust = TRUE, ci = 0.95, verbose = TRUE, ...) {
   r2_bayesian <- r2_posterior(model, verbose = verbose, ...)
 
   if (is.null(r2_bayesian)) {
@@ -88,20 +103,46 @@ r2_bayes <- function(model, robust = TRUE, ci = .95, verbose = TRUE, ...) {
   if (insight::is_multivariate(model)) {
     structure(
       class = "r2_bayes_mv",
-      rapply(r2_bayesian, ifelse(robust, stats::median, mean)),
-      "SE" = rapply(r2_bayesian, ifelse(robust, stats::mad, stats::sd)),
+      rapply(r2_bayesian, function(i) {
+        if (robust) {
+          stats::median(i)
+        } else {
+          mean(i)
+        }
+      }),
+      SE = rapply(r2_bayesian, function(i) {
+        if (robust) {
+          stats::mad(i)
+        } else {
+          stats::sd(i)
+        }
+      }),
       # "Estimates" = rapply(r2_bayesian, bayestestR::point_estimate, centrality = "all", dispersion = TRUE),
-      "CI" = rapply(r2_bayesian, bayestestR::hdi, ci = ci),
-      "robust" = robust
+      CI = rapply(r2_bayesian, bayestestR::hdi, ci = ci),
+      ci_method = "HDI",
+      robust = robust
     )
   } else {
     structure(
       class = "r2_bayes",
-      lapply(r2_bayesian, ifelse(robust, stats::median, mean)),
-      "SE" = lapply(r2_bayesian, ifelse(robust, stats::mad, stats::sd)),
-      # "Estimates" = lapply(r2_bayesian, bayestestR::point_estimate, centrality = "all", dispersion = TRUE),
-      "CI" = lapply(r2_bayesian, bayestestR::hdi, ci = ci),
-      "robust" = robust
+      lapply(r2_bayesian, function(i) {
+        if (robust) {
+          stats::median(i)
+        } else {
+          mean(i)
+        }
+      }),
+      SE = lapply(r2_bayesian, function(i) {
+        if (robust) {
+          stats::mad(i)
+        } else {
+          stats::sd(i)
+        }
+      }),
+      # Estimates = lapply(r2_bayesian, bayestestR::point_estimate, centrality = "all", dispersion = TRUE),
+      CI = lapply(r2_bayesian, bayestestR::hdi, ci = ci),
+      ci_method = "HDI",
+      robust = robust
     )
   }
 }
@@ -119,7 +160,9 @@ r2_posterior.brmsfit <- function(model, verbose = TRUE, ...) {
 
   algorithm <- insight::find_algorithm(model)
   if (algorithm$algorithm != "sampling") {
-    warning(insight::format_message("`r2()` only available for models fit using the 'sampling' algorithm."), call. = FALSE)
+    insight::format_warning(
+      "`r2()` only available for models fit using the `sampling` algorithm."
+    )
     return(NA)
   }
 
@@ -131,43 +174,43 @@ r2_posterior.brmsfit <- function(model, verbose = TRUE, ...) {
         res <- insight::find_response(model)
         if (mi[[1]]$is_mixed) {
           br2_mv <- list(
-            "R2_Bayes" = rstantools::bayes_R2(
+            R2_Bayes = rstantools::bayes_R2(
               model,
               re.form = NULL,
               re_formula = NULL,
               summary = FALSE
             ),
-            "R2_Bayes_marginal" = rstantools::bayes_R2(
+            R2_Bayes_marginal = rstantools::bayes_R2(
               model,
               re.form = NA,
               re_formula = NA,
               summary = FALSE
             )
           )
-          br2 <- lapply(1:length(res), function(x) {
+          br2 <- lapply(seq_along(res), function(x) {
             list(
-              "R2_Bayes" = unname(as.vector(br2_mv$R2_Bayes[, x])),
-              "R2_Bayes_marginal" = unname(as.vector(br2_mv$R2_Bayes_marginal[, x]))
+              R2_Bayes = unname(as.vector(br2_mv$R2_Bayes[, x])),
+              R2_Bayes_marginal = unname(as.vector(br2_mv$R2_Bayes_marginal[, x]))
             )
           })
           names(br2) <- res
         } else {
-          br2_mv <- list("R2_Bayes" = rstantools::bayes_R2(model, summary = FALSE))
-          br2 <- lapply(1:length(res), function(x) {
-            list("R2_Bayes" = unname(as.vector(br2_mv$R2_Bayes[, x])))
+          br2_mv <- list(R2_Bayes = rstantools::bayes_R2(model, summary = FALSE))
+          br2 <- lapply(seq_along(res), function(x) {
+            list(R2_Bayes = unname(as.vector(br2_mv$R2_Bayes[, x])))
           })
           names(br2) <- res
         }
       } else {
         if (mi$is_mixed) {
           br2 <- list(
-            "R2_Bayes" = as.vector(rstantools::bayes_R2(
+            R2_Bayes = as.vector(rstantools::bayes_R2(
               model,
               re.form = NULL,
               re_formula = NULL,
               summary = FALSE
             )),
-            "R2_Bayes_marginal" = as.vector(rstantools::bayes_R2(
+            R2_Bayes_marginal = as.vector(rstantools::bayes_R2(
               model,
               re.form = NA,
               re_formula = NA,
@@ -177,7 +220,7 @@ r2_posterior.brmsfit <- function(model, verbose = TRUE, ...) {
           names(br2$R2_Bayes) <- rep("Conditional R2", length(br2$R2_Bayes))
           names(br2$R2_Bayes_marginal) <- rep("Marginal R2", length(br2$R2_Bayes))
         } else {
-          br2 <- list("R2_Bayes" = as.vector(rstantools::bayes_R2(model, summary = FALSE)))
+          br2 <- list(R2_Bayes = as.vector(rstantools::bayes_R2(model, summary = FALSE)))
           names(br2$R2_Bayes) <- rep("R2", length(br2$R2_Bayes))
         }
       }
@@ -202,7 +245,7 @@ r2_posterior.stanreg <- r2_posterior.brmsfit
 #' @export
 r2_posterior.stanmvreg <- function(model, verbose = TRUE, ...) {
   if (isTRUE(verbose)) {
-    warning("Models of class 'stanmvreg' not yet supported.", call. = FALSE)
+    insight::format_error("Models of class `stanmvreg` not yet supported.")
   }
   NULL
 }
@@ -221,7 +264,7 @@ r2_posterior.BFBayesFactor <- function(model,
   mi <- insight::model_info(model, verbose = FALSE)
   if (!mi$is_linear || mi$is_correlation || mi$is_ttest || mi$is_binomial || mi$is_meta) {
     if (verbose) {
-      warning("Can produce R2 only for linear models.", call. = FALSE)
+      insight::format_warning("Can produce R2 only for linear models.")
     }
     return(NULL)
   }
@@ -233,41 +276,16 @@ r2_posterior.BFBayesFactor <- function(model,
   insight::check_if_installed("rstantools")
   insight::check_if_installed("BayesFactor")
 
-  # Estimates
-  params <- insight::get_parameters(model, unreduce = FALSE)
-
-  # remove sig and g cols
-  params <- params[, !grepl(pattern = "^sig2$|^g_|^g$", colnames(params))]
-
-  # Model Matrix
-  mm <- BayesFactor::model.matrix(model[1])
-  colnames(mm)[1] <- "mu"
-
-  # match?
-  if ((length(colnames(params)) != length(colnames(mm))) ||
-    !all(colnames(params) == colnames(mm))) {
-    if (utils::packageVersion("BayesFactor") < package_version("0.9.12.4.3")) {
-      stop("R2 for BayesFactor models with random effects requires BayesFactor v0.9.12.4.3 or higher.", call. = FALSE)
-    }
-    stop("Woops, you seem to have stumbled on some weird edge case. Please file an issue at https://github.com/easystats/performance/issues", call. = FALSE)
-  }
+  everything_we_need <- .get_bfbf_predictions(model, verbose = verbose)
 
   # Compute R2!
-  y <- insight::get_response(model, verbose = FALSE)
-  yy <- as.matrix(params) %*% t(mm)
-  r2s <- rstantools::bayes_R2(yy, y = y)
-  r2_bayesian <- data.frame(R2_Bayes = r2s)
+  y <- everything_we_need[["y"]]
+  yy <- everything_we_need[["y_pred"]]
+  r2_bayesian <- data.frame(R2_Bayes = rstantools::bayes_R2(yy, y = y))
 
-  rand <- insight::find_predictors(model[1], effects = "random", flatten = TRUE, verbose = FALSE)
-
-  if (!is.null(rand)) {
-    idx <- sapply(paste0("\\b", rand, "\\b"), grepl, x = colnames(params))
-    idx <- apply(idx, 1, any)
-    params[idx] <- 0
-
-    yy <- as.matrix(params) %*% t(mm)
-    r2s_marginal <- rstantools::bayes_R2(yy, y = y)
-    r2_bayesian$R2_Bayes_marginal <- r2s_marginal
+  if ("y_pred_marginal" %in% names(everything_we_need)) {
+    yy <- everything_we_need[["y_pred_marginal"]]
+    r2_bayesian$R2_Bayes_marginal <- rstantools::bayes_R2(yy, y = y)
   }
 
   r2_bayesian
@@ -289,7 +307,9 @@ r2_posterior.BFBayesFactor <- function(model,
 
   if (any(is.na(BFMods$BF) | is.infinite(BFMods$BF))) {
     if (verbose) {
-      warning(insight::format_message("Can't compute model-averaged index. One or more Bayes factors are NA or infinite."), call. = FALSE)
+      insight::format_warning(
+        "Can't compute model-averaged index. One or more Bayes factors are NA or infinite."
+      )
     }
     return(NULL)
   }
@@ -315,10 +335,10 @@ r2_posterior.BFBayesFactor <- function(model,
 
 
   # Compute posterior model probabilities
-  if (!is.null(prior_odds)) {
-    prior_odds <- c(1, prior_odds)
-  } else {
+  if (is.null(prior_odds)) {
     prior_odds <- rep(1, nrow(BFMods))
+  } else {
+    prior_odds <- c(1, prior_odds)
   }
   posterior_odds <- prior_odds * BFMods$BF
   posterior_odds <- posterior_odds[-1] / posterior_odds[1]
@@ -339,6 +359,7 @@ as.data.frame.r2_bayes <- function(x, ...) {
     CI = attributes(x)$CI$R2_Bayes$CI,
     CI_low = attributes(x)$CI$R2_Bayes$CI_low,
     CI_high = attributes(x)$CI$R2_Bayes$CI_high,
+    CI_method = attributes(x)$ci_method,
     stringsAsFactors = FALSE
   )
 
@@ -349,6 +370,7 @@ as.data.frame.r2_bayes <- function(x, ...) {
       CI = attributes(x)$CI$R2_Bayes_marginal$CI,
       CI_low = attributes(x)$CI$R2_Bayes_marginal$CI_low,
       CI_high = attributes(x)$CI$R2_Bayes_marginal$CI_high,
+      CI_method = attributes(x)$ci_method,
       stringsAsFactors = FALSE
     )
 
@@ -357,5 +379,68 @@ as.data.frame.r2_bayes <- function(x, ...) {
     out <- rbind(out, out_marginal)
   }
 
+  out$Effectsize <- "Bayesian R-squared"
   out
 }
+
+
+
+# Utils -------------------------------------------------------------------
+
+.get_bfbf_predictions <- function(model, iterations = 4000, verbose = TRUE) {
+  insight::check_if_installed("BayesFactor")
+
+  # Estimates
+  params <- insight::get_parameters(
+    model,
+    unreduce = FALSE,
+    iterations = iterations,
+    verbose = verbose
+  )
+
+  # remove sig and g cols
+  params_theta <- params[, !grepl(pattern = "^sig2$|^g_|^g$", colnames(params))]
+  params_sigma <- sqrt(params[, grepl(pattern = "^sig2$", colnames(params))])
+
+  # Model Matrix
+  mm <- insight::get_modelmatrix(model[1])
+  colnames(mm)[1] <- "mu"
+
+  # match?
+  if ((length(colnames(params_theta)) != length(colnames(mm))) ||
+    !all(colnames(params_theta) == colnames(mm))) {
+    if (utils::packageVersion("BayesFactor") < package_version("0.9.12.4.3")) {
+      insight::format_error("R2 for BayesFactor models with random effects requires BayesFactor v0.9.12.4.3 or higher.")
+    }
+    insight::format_error("Woops, you seem to have stumbled on some weird edge case. Please file an issue at {.url https://github.com/easystats/performance/issues}") # nolint
+  }
+
+  out <- list(
+    y = insight::get_response(model, verbose = FALSE),
+    y_pred = (as.matrix(params_theta) %*% t(mm))
+  )
+
+  rand <- insight::find_predictors(model[1], effects = "random", flatten = TRUE, verbose = FALSE)
+  if (!is.null(rand)) {
+    idx <- sapply(paste0("\\b", rand, "\\b"), grepl, x = colnames(params_theta))
+    idx <- apply(idx, 1, any)
+    params_theta[idx] <- 0
+
+    out[["y_pred_marginal"]] <- (as.matrix(params_theta) %*% t(mm))
+  }
+
+  out[["sigma"]] <- params_sigma
+  out
+}
+
+#' @export
+residuals.BFBayesFactor <- function(object, ...) {
+  everything_we_need <- .get_bfbf_predictions(object, verbose = FALSE)
+
+  everything_we_need[["y"]] - colMeans(everything_we_need[["y_pred"]])
+}
+
+#' @export
+fitted.BFBayesFactor <- function(object, ...) {
+  colMeans(.get_bfbf_predictions(object, verbose = FALSE)[["y_pred"]])
+}
