@@ -1,18 +1,18 @@
 #' @title McKelvey & Zavoinas R2
 #' @name r2_mckelvey
 #'
-#' @description Calculates McKelvey & Zavoinas pseudo R2.
+#' @description Calculates McKelvey and Zavoinas pseudo R2.
 #'
 #' @param model Generalized linear model.
 #'
 #' @return The R2 value.
 #'
 #' @references
-#' \itemize{
-#'   \item McKelvey, R., Zavoina, W. (1975), "A Statistical Model for the Analysis of Ordinal Level Dependent Variables", Journal of Mathematical Sociology 4, S. 103–120.
-#' }
+#' McKelvey, R., Zavoina, W. (1975), "A Statistical Model for the Analysis of
+#' Ordinal Level Dependent Variables", Journal of Mathematical Sociology 4, S.
+#' 103–120.
 #'
-#' @details McKelvey & Zavoinas R2 is based on the explained variance,
+#' @details McKelvey and Zavoinas R2 is based on the explained variance,
 #'   where the variance of the predicted response is divided by the sum
 #'   of the variance of the predicted response and residual variance.
 #'   For binomial models, the residual variance is either `pi^2/3`
@@ -21,7 +21,6 @@
 #'   the *distribution-specific variance* as described in
 #'   `?insight::get_variance`.
 #'
-#'
 #' @examples
 #' ## Dobson (1990) Page 93: Randomized Controlled Trial:
 #' counts <- c(18, 17, 15, 20, 10, 20, 25, 13, 12) #
@@ -44,9 +43,9 @@ r2_mckelvey.default <- function(model) {
 
 .r2_mckelvey <- function(model) {
   faminfo <- insight::model_info(model)
-  n <- insight::n_obs(model)
+  n <- insight::n_obs(model, disaggregate = TRUE)
 
-  if (faminfo$is_binomial | faminfo$is_ordinal | faminfo$is_multinomial) {
+  if (faminfo$is_binomial || faminfo$is_ordinal || faminfo$is_multinomial) {
     dist.variance <- switch(faminfo$link_function,
       probit = 1,
       logit = pi^2 / 3,
