@@ -44,16 +44,21 @@
 #'  - Nakagawa, S., Johnson, P. C. D., and Schielzeth, H. (2017). The
 #'    coefficient of determination R2 and intra-class correlation coefficient from
 #'    generalized linear mixed-effects models revisited and expanded. Journal of
-#'    The Royal Society Interface, 14(134), 20170213. \doi{10.1098/rsif.2017.0213}
+#'    The Royal Society Interface, 14(134), 20170213.
 #'
-#' @examples
-#' if (require("lme4")) {
-#'   model <- lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
-#'   r2_nakagawa(model)
-#'   r2_nakagawa(model, by_group = TRUE)
-#' }
+#' @examplesIf require("lme4")
+#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
+#' r2_nakagawa(model)
+#' r2_nakagawa(model, by_group = TRUE)
 #' @export
-r2_nakagawa <- function(model, by_group = FALSE, tolerance = 1e-5, ci = NULL, iterations = 100, ...) {
+r2_nakagawa <- function(model,
+                        by_group = FALSE,
+                        tolerance = 1e-5,
+                        ci = NULL,
+                        iterations = 100,
+                        ci_method = NULL,
+                        verbose = TRUE,
+                        ...) {
   # calculate random effect variances
   vars <- .compute_random_vars(
     model,
@@ -71,11 +76,11 @@ r2_nakagawa <- function(model, by_group = FALSE, tolerance = 1e-5, ci = NULL, it
   # compute R2 by group
   if (isTRUE(by_group)) {
     # with random slopes, explained variance is inaccurate
-    if (!is.null(insight::find_random_slopes(model))) {
+    if (!is.null(insight::find_random_slopes(model)) && verbose) {
       insight::format_warning("Model contains random slopes. Explained variance by levels is not accurate.")
     }
 
-    if (!is.null(ci) && !is.na(ci)) {
+    if (!is.null(ci) && !is.na(ci) && verbose) {
       insight::format_warning("Confidence intervals are not yet supported for `by_group = TRUE`.")
     }
 
@@ -100,8 +105,10 @@ r2_nakagawa <- function(model, by_group = FALSE, tolerance = 1e-5, ci = NULL, it
   } else {
     # Calculate R2 values
     if (insight::is_empty_object(vars$var.random) || is.na(vars$var.random)) {
-      # if no random effect variance, return simple R2
-      insight::print_color("Random effect variances not available. Returned R2 does not account for random effects.\n", "red")
+      if (verbose) {
+        # if no random effect variance, return simple R2
+        insight::print_color("Random effect variances not available. Returned R2 does not account for random effects.\n", "red")
+      }
       r2_marginal <- vars$var.fixed / (vars$var.fixed + vars$var.residual)
       r2_conditional <- NA
     } else {
@@ -115,18 +122,54 @@ r2_nakagawa <- function(model, by_group = FALSE, tolerance = 1e-5, ci = NULL, it
 
     # check if CIs are requested, and compute bootstrapped CIs
     if (!is.null(ci) && !is.na(ci)) {
-      result <- .bootstrap_r2_nakagawa(model, iterations, tolerance, ...)
-      # CI for marginal R2
-      r2_ci <- as.vector(result$t[, 1])
-      r2_ci <- r2_ci[!is.na(r2_ci)]
-      r2_ci <- bayestestR::eti(r2_ci, ci = ci)
-      out$R2_marginal <- c(out$R2_marginal, CI_low = r2_ci$CI_low, CI_high = r2_ci$CI_high)
+      # this is experimental!
+      if (identical(ci_method, "analytical")) {
+        result <- .safe(.analytical_icc_ci(model, ci, fun = "r2_nakagawa"))
+        if (!is.null(result)) {
+          r2_ci_marginal <- result$R2_marginal
+          r2_ci_conditional <- result$R2_conditional
+        } else {
+          r2_ci_marginal <- r2_ci_conditional <- NA
+        }
+      } else {
+        result <- .bootstrap_r2_nakagawa(model, iterations, tolerance, ci_method, ...)
+        # CI for marginal R2
+        r2_ci_marginal <- as.vector(result$t[, 1])
+        r2_ci_marginal <- r2_ci_marginal[!is.na(r2_ci_marginal)]
+        # validation check
+        if (length(r2_ci_marginal) > 0) {
+          r2_ci_marginal <- bayestestR::eti(r2_ci_marginal, ci = ci)
+        } else {
+          r2_ci_marginal <- NA
+        }
 
-      # CI for unadjusted R2
-      r2_ci <- as.vector(result$t[, 2])
-      r2_ci <- r2_ci[!is.na(r2_ci)]
-      r2_ci <- bayestestR::eti(r2_ci, ci = ci)
-      out$R2_conditional <- c(out$R2_conditional, CI_low = r2_ci$CI_low, CI_high = r2_ci$CI_high)
+        # CI for unadjusted R2
+        r2_ci_conditional <- as.vector(result$t[, 2])
+        r2_ci_conditional <- r2_ci_conditional[!is.na(r2_ci_conditional)]
+        # validation check
+        if (length(r2_ci_conditional) > 0) {
+          r2_ci_conditional <- bayestestR::eti(r2_ci_conditional, ci = ci)
+        } else {
+          r2_ci_conditional <- NA
+        }
+
+        if ((all(is.na(r2_ci_marginal)) || all(is.na(r2_ci_conditional))) && verbose) {
+          insight::format_warning(
+            "Could not compute confidence intervals for R2. Try `ci_method = \"simple\"."
+          )
+        }
+      }
+
+      out$R2_marginal <- c(
+        out$R2_marginal,
+        CI_low = r2_ci_marginal$CI_low,
+        CI_high = r2_ci_marginal$CI_high
+      )
+      out$R2_conditional <- c(
+        out$R2_conditional,
+        CI_low = r2_ci_conditional$CI_low,
+        CI_high = r2_ci_conditional$CI_high
+      )
 
       attr(out, "ci") <- ci
     }
@@ -224,8 +267,8 @@ print.r2_nakagawa <- function(x, digits = 3, ...) {
 }
 
 # main bootstrap function
-.bootstrap_r2_nakagawa <- function(model, iterations, tolerance, ...) {
-  if (inherits(model, c("merMod", "lmerMod", "glmmTMB"))) {
+.bootstrap_r2_nakagawa <- function(model, iterations, tolerance, ci_method = NULL, ...) {
+  if (inherits(model, c("merMod", "lmerMod", "glmmTMB")) && !identical(ci_method, "boot")) {
     result <- .do_lme4_bootmer(
       model,
       .boot_r2_fun_lme4,
