@@ -1,4 +1,4 @@
-requiet("ISLR")
+skip_if_not_installed("ISLR")
 data(Smarket, package = "ISLR")
 
 m1 <- glm(am ~ vs + wt, family = binomial(), data = mtcars)
@@ -7,8 +7,8 @@ m2 <- glm(Direction ~ Lag1 + Volume, family = binomial(), data = Smarket)
 roc1 <- performance_roc(m1)
 roc2 <- performance_roc(m2)
 
-auc1 <- area_under_curve(roc1$Specificity, roc1$Sensitivity)
-auc2 <- area_under_curve(roc2$Specificity, roc2$Sensitivity)
+auc1 <- bayestestR::area_under_curve(roc1$Specificity, roc1$Sensitivity)
+auc2 <- bayestestR::area_under_curve(roc2$Specificity, roc2$Sensitivity)
 
 test_that("roc", {
   expect_equal(head(roc1$Sensitivity), c(0, 0.07692, 0.15385, 0.23077, 0.30769, 0.38462), tolerance = 1e-2)
